// Copyright 2017-2025 WIT.COM Inc. All rights reserved.
// Use of this source code is governed by the GPL 3.0

package main

// An app to submit patches for the 30 GO GUI repos

import (
	"os"
	"time"

	"go.wit.com/gui"
	"go.wit.com/lib/gadgets"
	"go.wit.com/lib/protobuf/zoopb"
	"go.wit.com/log"
)

// refresh the windows & tables the user has open
func refresh() {
	time.Sleep(90 * time.Second)
	if argv.Verbose {
		log.Info("zookeeper scan here")
	}
	if me.zood != nil {
		me.zood.doMachinesUpgradeTable(me.machines)
	}
}

func doGui() {
	me.myGui = gui.New()
	me.myGui.InitEmbed(resources)
	me.myGui.Default()

	win := gadgets.RawBasicWindow("Zookeeper: (inventory your cluster)")
	win.Make()
	win.Show()
	win.Custom = func() {
		log.Warn("Main window close")
		os.Exit(0)
	}

	box := win.Box()

	vbox := box.NewVerticalBox("BOX2")

	group1 := vbox.NewGroup("Zookeeper Settings")
	grid := group1.NewGrid("buildOptions", 0, 0)

	grid.NewButton("zood versions", func() {
		// if the window exists, just toggle it open or closed
		if me.zood != nil {
			me.zood.Toggle()
			return
		}
		makeZoodWin()
	})

	grid.NewButton("Cluster Events", func() {
		log.Info("todo: start a list here!")
	})

	// sit here forever refreshing the GUI
	for {
		refresh()
	}
}

func saveMachineState() {
	cur := zoopb.NewMachines()

	all := me.machines.SortByHostname()
	for all.Scan() {
		m := all.Next()
		log.Info("have machine:", m.Hostname)
		cur.Append(m)
	}
	cur.ConfigSave()
}
